@@ -35,9 +35,10 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         bool realloc_lhs_, bool allocate_target_) :
     al(al_), pass_result(pass_result_),
     remove_original_statement(remove_original_statement_),
-    current_scope(nullptr), result_var(nullptr), result_counter(0),
-    resultvar2value(resultvar2value_), realloc_lhs(realloc_lhs_),
-    allocate_target(allocate_target_) {}
+    current_scope(nullptr), 
+    result_var(nullptr), result_counter(0),
+    resultvar2value(resultvar2value_), 
+    realloc_lhs(realloc_lhs_), allocate_target(allocate_target_) {}
 
     ASR::expr_t* get_ImpliedDoLoop_size(ASR::ImpliedDoLoop_t* implied_doloop) {
         const Location& loc = implied_doloop->base.base.loc;
@@ -47,14 +48,15 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         ASR::expr_t* d = implied_doloop->m_increment;
         ASR::expr_t* implied_doloop_size = nullptr;
         int kind = ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(end));
+        start = builder.i2i_t(start, ASRUtils::expr_type(end));
         if( d == nullptr ) {
-            implied_doloop_size = builder.ElementalAdd(
-                builder.ElementalSub(end, start, loc),
-                make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, kind, loc), loc);
+            implied_doloop_size = builder.Add(
+                builder.Sub(end, start),
+                make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, kind, loc));
         } else {
-            implied_doloop_size = builder.ElementalAdd(builder.ElementalDiv(
-                builder.ElementalSub(end, start, loc), d, loc),
-                make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, kind, loc), loc);
+            implied_doloop_size = builder.Add(builder.Div(
+                builder.Sub(end, start), d),
+                make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, kind, loc));
         }
         int const_elements = 0;
         ASR::expr_t* implied_doloop_size_ = nullptr;
@@ -64,9 +66,9 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                     implied_doloop_size_ = get_ImpliedDoLoop_size(
                         ASR::down_cast<ASR::ImpliedDoLoop_t>(implied_doloop->m_values[i]));
                 } else {
-                    implied_doloop_size_ = builder.ElementalAdd(get_ImpliedDoLoop_size(
+                    implied_doloop_size_ = builder.Add(get_ImpliedDoLoop_size(
                         ASR::down_cast<ASR::ImpliedDoLoop_t>(implied_doloop->m_values[i])),
-                        implied_doloop_size_, loc);
+                        implied_doloop_size_);
                 }
             } else {
                 const_elements += 1;
@@ -77,32 +79,23 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 implied_doloop_size_ = make_ConstantWithKind(make_IntegerConstant_t,
                     make_Integer_t, const_elements, kind, loc);
             } else {
-                implied_doloop_size_ = builder.ElementalAdd(
+                implied_doloop_size_ = builder.Add(
                     make_ConstantWithKind(make_IntegerConstant_t,
                         make_Integer_t, const_elements, kind, loc),
-                    implied_doloop_size_, loc);
+                    implied_doloop_size_);
             }
         }
         if( implied_doloop_size_ ) {
-            implied_doloop_size = builder.ElementalMul(implied_doloop_size_, implied_doloop_size, loc);
+            implied_doloop_size = builder.Mul(implied_doloop_size_, implied_doloop_size);
         }
         return implied_doloop_size;
     }
 
     size_t get_constant_ArrayConstant_size(ASR::ArrayConstant_t* x) {
-        size_t size = 0;
-        for( size_t i = 0; i < x->n_args; i++ ) {
-            if( ASR::is_a<ASR::ArrayConstant_t>(*x->m_args[i]) ) {
-                size += get_constant_ArrayConstant_size(
-                    ASR::down_cast<ASR::ArrayConstant_t>(x->m_args[i]));
-            } else {
-                size += 1;
-            }
-        }
-        return size;
+        return ASRUtils::get_fixed_size_of_array(x->m_type);
     }
 
-    ASR::expr_t* get_ArrayConstant_size(ASR::ArrayConstant_t* x, bool& is_allocatable) {
+    ASR::expr_t* get_ArrayConstructor_size(ASR::ArrayConstructor_t* x, bool& is_allocatable) {
         ASR::ttype_t* int_type = ASRUtils::TYPE(ASR::make_Integer_t(al, x->base.base.loc, 4));
         ASR::expr_t* array_size = nullptr;
         int64_t constant_size = 0;
@@ -116,14 +109,23 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                         ASR::down_cast<ASR::ArrayConstant_t>(element));
                 } else {
                     ASR::expr_t* element_array_size = get_ArrayConstant_size(
-                        ASR::down_cast<ASR::ArrayConstant_t>(element), is_allocatable);
+                        ASR::down_cast<ASR::ArrayConstant_t>(element));
                     if( array_size == nullptr ) {
                         array_size = element_array_size;
                     } else {
-                        array_size = builder.ElementalAdd(array_size,
-                                        element_array_size, x->base.base.loc);
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
                     }
                 }
+            } else if( ASR::is_a<ASR::ArrayConstructor_t>(*element) ) {
+                ASR::expr_t* element_array_size = get_ArrayConstructor_size(
+                    ASR::down_cast<ASR::ArrayConstructor_t>(element), is_allocatable);
+                if( array_size == nullptr ) {
+                    array_size = element_array_size;
+                } else {
+                    array_size = builder.Add(array_size,
+                                    element_array_size);
+                }
             } else if( ASR::is_a<ASR::Var_t>(*element) ) {
                 ASR::ttype_t* element_type = ASRUtils::type_get_past_allocatable(
                     ASRUtils::expr_type(element));
@@ -137,8 +139,8 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                         if( array_size == nullptr ) {
                             array_size = element_array_size;
                         } else {
-                            array_size = builder.ElementalAdd(array_size,
-                                            element_array_size, x->base.base.loc);
+                            array_size = builder.Add(array_size,
+                                            element_array_size);
                         }
                     }
                 } else {
@@ -148,7 +150,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 ASR::expr_t* implied_doloop_size = get_ImpliedDoLoop_size(
                     ASR::down_cast<ASR::ImpliedDoLoop_t>(element));
                 if( array_size ) {
-                    array_size = builder.ElementalAdd(implied_doloop_size, array_size, loc);
+                    array_size = builder.Add(implied_doloop_size, array_size);
                 } else {
                     array_size = implied_doloop_size;
                 }
@@ -162,19 +164,19 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                     if( d == nullptr ) {
                         continue;
                     }
-                    ASR::expr_t* dim_size = builder.ElementalAdd(builder.ElementalDiv(
-                        builder.ElementalSub(end, start, loc), d, loc),
-                        make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, 4, loc), loc);
+                    ASR::expr_t* dim_size = builder.Add(builder.Div(
+                        builder.Sub(end, start), d),
+                        make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, 4, loc));
                     if( array_section_size == nullptr ) {
                         array_section_size = dim_size;
                     } else {
-                        array_section_size = builder.ElementalMul(array_section_size, dim_size, loc);
+                        array_section_size = builder.Mul(array_section_size, dim_size);
                     }
                 }
                 if( array_size == nullptr ) {
                     array_size = array_section_size;
                 } else {
-                    builder.ElementalAdd(array_section_size, array_size, loc);
+                    array_size = builder.Add(array_section_size, array_size);
                 }
             } else {
                 constant_size += 1;
@@ -192,7 +194,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
             }
         }
         if( constant_size_asr ) {
-            array_size = builder.ElementalAdd(array_size, constant_size_asr, x->base.base.loc);
+            array_size = builder.Add(array_size, constant_size_asr);
         }
         is_allocatable = true;
         if( array_size == nullptr ) {
@@ -202,6 +204,103 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         return array_size;
     }
 
+    ASR::expr_t* get_ArrayConstant_size(ASR::ArrayConstant_t* x) {
+        ASR::ttype_t* int_type = ASRUtils::TYPE(ASR::make_Integer_t(al, x->base.base.loc, 4));
+        return make_ConstantWithType(make_IntegerConstant_t,
+                ASRUtils::get_fixed_size_of_array(x->m_type), int_type, x->base.base.loc);
+    }
+
+   void replace_ArrayConstructor(ASR::ArrayConstructor_t* x) {
+        const Location& loc = x->base.base.loc;
+        ASR::expr_t* result_var_copy = result_var;
+        bool is_result_var_fixed_size = false;
+        if (result_var != nullptr &&
+            resultvar2value.find(result_var) != resultvar2value.end() &&
+            resultvar2value[result_var] == &(x->base)) {
+            is_result_var_fixed_size = ASRUtils::is_fixed_size_array(ASRUtils::expr_type(result_var));
+        }
+        ASR::ttype_t* result_type_ = nullptr;
+        bool is_allocatable = false;
+        ASR::expr_t* array_constructor = get_ArrayConstructor_size(x, is_allocatable);
+        Vec<ASR::dimension_t> dims;
+        dims.reserve(al, 1);
+        ASR::dimension_t dim;
+        dim.loc = loc;
+        dim.m_start = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1,
+                        ASRUtils::type_get_past_pointer(
+                            ASRUtils::type_get_past_allocatable(
+                                ASRUtils::expr_type(array_constructor)))));
+        dim.m_length = array_constructor;
+        dims.push_back(al, dim);
+        remove_original_statement = false;
+        if( is_result_var_fixed_size ) {
+            result_type_ = ASRUtils::expr_type(result_var);
+            is_allocatable = false;
+        } else {
+            if( is_allocatable ) {
+                result_type_ = ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->m_type->base.loc,
+                    ASRUtils::type_get_past_allocatable(
+                        ASRUtils::duplicate_type_with_empty_dims(al, x->m_type))));
+            } else {
+                result_type_ = ASRUtils::duplicate_type(al,
+                    ASRUtils::type_get_past_allocatable(x->m_type), &dims);
+            }
+        }
+        result_var = PassUtils::create_var(result_counter, "_array_constructor_",
+                        loc, result_type_, al, current_scope);
+        result_counter += 1;
+        *current_expr = result_var;
+
+        Vec<ASR::alloc_arg_t> alloc_args;
+        alloc_args.reserve(al, 1);
+        ASR::alloc_arg_t arg;
+        arg.m_len_expr = nullptr;
+        arg.m_type = nullptr;
+        arg.m_dims = dims.p;
+        arg.n_dims = dims.size();
+        if( is_allocatable ) {
+            arg.loc = result_var->base.loc;
+            arg.m_a = result_var;
+            alloc_args.push_back(al, arg);
+            Vec<ASR::expr_t*> to_be_deallocated;
+            to_be_deallocated.reserve(al, alloc_args.size());
+            for( size_t i = 0; i < alloc_args.size(); i++ ) {
+                to_be_deallocated.push_back(al, alloc_args.p[i].m_a);
+            }
+            pass_result.push_back(al, ASRUtils::STMT(ASR::make_ExplicitDeallocate_t(
+                al, loc, to_be_deallocated.p, to_be_deallocated.size())));
+            ASR::stmt_t* allocate_stmt = ASRUtils::STMT(ASR::make_Allocate_t(
+                al, loc, alloc_args.p, alloc_args.size(), nullptr, nullptr, nullptr));
+            pass_result.push_back(al, allocate_stmt);
+        }
+        if ( allocate_target && realloc_lhs ) {
+            allocate_target = false;
+            arg.loc = result_var_copy->base.loc;
+            arg.m_a = result_var_copy;
+            alloc_args.push_back(al, arg);
+            Vec<ASR::expr_t*> to_be_deallocated;
+            to_be_deallocated.reserve(al, alloc_args.size());
+            for( size_t i = 0; i < alloc_args.size(); i++ ) {
+                to_be_deallocated.push_back(al, alloc_args.p[i].m_a);
+            }
+            pass_result.push_back(al, ASRUtils::STMT(ASR::make_ExplicitDeallocate_t(
+                al, loc, to_be_deallocated.p, to_be_deallocated.size())));
+            ASR::stmt_t* allocate_stmt = ASRUtils::STMT(ASR::make_Allocate_t(
+                al, loc, alloc_args.p, alloc_args.size(), nullptr, nullptr, nullptr));
+            pass_result.push_back(al, allocate_stmt);
+        }
+        LCOMPILERS_ASSERT(result_var != nullptr);
+        Vec<ASR::stmt_t*>* result_vec = &pass_result;
+        if (ASRUtils::use_experimental_simplifier) {
+            PassUtils::ReplacerUtils::replace_ArrayConstructor_(al, x, result_var,
+                result_vec, current_scope);
+        } else {
+            PassUtils::ReplacerUtils::replace_ArrayConstructor(x, this,
+                remove_original_statement, result_vec);
+        }
+        result_var = result_var_copy;
+    }
+
     void replace_ArrayConstant(ASR::ArrayConstant_t* x) {
         const Location& loc = x->base.base.loc;
         ASR::expr_t* result_var_copy = result_var;
@@ -213,7 +312,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         }
         ASR::ttype_t* result_type_ = nullptr;
         bool is_allocatable = false;
-        ASR::expr_t* array_constant_size = get_ArrayConstant_size(x, is_allocatable);
+        ASR::expr_t* array_constant_size = get_ArrayConstant_size(x);
         Vec<ASR::dimension_t> dims;
         dims.reserve(al, 1);
         ASR::dimension_t dim;
@@ -230,7 +329,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
             is_allocatable = false;
         } else {
             if( is_allocatable ) {
-                result_type_ = ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->m_type->base.loc,
+                result_type_ = ASRUtils::TYPE(ASRUtils::make_Allocatable_t_util(al, x->m_type->base.loc,
                     ASRUtils::type_get_past_allocatable(
                         ASRUtils::duplicate_type_with_empty_dims(al, x->m_type))));
             } else {
@@ -319,6 +418,8 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
 
         Allocator& al;
         bool remove_original_statement, allocate_target = false;
+        bool print = false, file_write = false;
+        ASR::expr_t* m_unit = nullptr;
         ReplaceArrayConstant replacer;
         Vec<ASR::stmt_t*> pass_result;
         Vec<ASR::stmt_t*>* parent_body;
@@ -333,6 +434,8 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
         parent_body(nullptr) {
             pass_result.n = 0;
             pass_result.reserve(al, 0);
+            print = false, file_write = false;
+            m_unit = nullptr;
         }
 
         void visit_Variable(const ASR::Variable_t& /*x*/) {
@@ -428,12 +531,50 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
             dim.push_back(al, d);
 
             ASR::ttype_t* array_type = ASRUtils::TYPE(ASR::make_Array_t(al, value->base.loc, ASRUtils::expr_type(value), dim.p, dim.size(), ASR::array_physical_typeType::FixedSizeArray));
-            ASR::asr_t* array_constant = ASR::make_ArrayConstant_t(al, value->base.loc,
+            ASR::asr_t* array_constant = ASRUtils::make_ArrayConstructor_t_util(al, value->base.loc,
                                         args.p, args.n, array_type, ASR::arraystorageType::ColMajor);
             return array_constant;
         }
 
+        ASR::stmt_t* create_do_loop_form_idl(ASR::ImpliedDoLoop_t* x) {
+            ASR::stmt_t* do_loop = nullptr;
+
+            ASR::do_loop_head_t do_loop_head;
+            do_loop_head.loc = x->base.base.loc; do_loop_head.m_v = x->m_var;
+            do_loop_head.m_start = x->m_start; do_loop_head.m_end = x->m_end;
+            do_loop_head.m_increment = x->m_increment;
+
+            Vec<ASR::stmt_t*> do_loop_body; do_loop_body.reserve(al, x->n_values);
+            for (size_t i = 0; i < x->n_values; i++ ) {
+                if ( ASR::is_a<ASR::ImpliedDoLoop_t>(*x->m_values[i]) ) {
+                    do_loop_body.push_back(al, create_do_loop_form_idl(
+                        ASR::down_cast<ASR::ImpliedDoLoop_t>(x->m_values[i])));
+                } else {
+                    Vec<ASR::expr_t*> print_values; print_values.reserve(al, 1);
+                    ASR::stmt_t* stmt = nullptr;
+                    Vec<ASR::expr_t*> args;
+                    args.reserve(al, 1);
+                    args.push_back(al, x->m_values[i]);
+                    ASR::expr_t* fmt_val = ASRUtils::EXPR(ASR::make_StringFormat_t(al,x->base.base.loc, nullptr,
+                        args.p, 1,ASR::string_format_kindType::FormatFortran, 
+                        ASRUtils::TYPE(ASR::make_String_t(al,x->base.base.loc,-1,-1,nullptr, ASR::string_physical_typeType::PointerString)), nullptr));
+                    print_values.push_back(al, fmt_val);
+                    if ( print ) {
+                        stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, x->m_values[i]->base.loc, print_values.p, print_values.size()));
+                    } else {
+                        // this will be file_write
+                        LCOMPILERS_ASSERT(file_write);
+                        stmt = ASRUtils::STMT(ASR::make_FileWrite_t(al, x->m_values[i]->base.loc, 0, m_unit, nullptr, nullptr, nullptr, print_values.p, print_values.size(), nullptr, nullptr, nullptr));
+                    }
+                    do_loop_body.push_back(al, stmt);
+                }
+            }
+            do_loop = ASRUtils::STMT(ASR::make_DoLoop_t(al, x->base.base.loc, nullptr, do_loop_head, do_loop_body.p, do_loop_body.size(), nullptr, 0));
+            return do_loop;
+        }
+
         void visit_Print(const ASR::Print_t &x) {
+            print = true;
             /*
                 integer :: i
                 print *, (i, i=1, 10)
@@ -443,31 +584,18 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                 integer :: i
                 print *, [(i, i=1, 10)]
             */
-            ASR::Print_t* print_stmt = const_cast<ASR::Print_t*>(&x);
-            for(size_t i = 0; i < x.n_values; i++) {
-                ASR::expr_t* value = x.m_values[i];
-                if (ASR::is_a<ASR::ImpliedDoLoop_t>(*value)) {
-                    ASR::asr_t* array_constant = create_array_constant(x, value);
-                    print_stmt->m_values[i] = ASRUtils::EXPR(array_constant);
-
-                    replacer.result_var = value;
-                    resultvar2value[replacer.result_var] = ASRUtils::EXPR(array_constant);
-                    ASR::expr_t** current_expr_copy_9 = current_expr;
-                    current_expr = const_cast<ASR::expr_t**>(&(print_stmt->m_values[i]));
-                    this->call_replacer();
-                    current_expr = current_expr_copy_9;
-                    if( !remove_original_statement ) {
-                        this->visit_expr(*print_stmt->m_values[i]);
-                    }
-                } else {
-                    ASR::expr_t** current_expr_copy_9 = current_expr;
-                    current_expr = const_cast<ASR::expr_t**>(&(print_stmt->m_values[i]));
-                    this->call_replacer();
-                    current_expr = current_expr_copy_9;
-                    if( !remove_original_statement ) {
-                        this->visit_expr(*print_stmt->m_values[i]);
-                    }
+            if(ASR::is_a<ASR::String_t>(*ASRUtils::expr_type(x.m_text))){
+                ASR::Print_t* print_stmt = const_cast<ASR::Print_t*>(&x);
+                ASR::expr_t** current_expr_copy_9 = current_expr;
+                current_expr = const_cast<ASR::expr_t**>(&(print_stmt->m_text));
+                this->call_replacer();
+                current_expr = current_expr_copy_9;
+                if( !remove_original_statement ) {
+                    this->visit_expr(*print_stmt->m_text);
                 }
+                print = false;
+            } else {
+                LCOMPILERS_ASSERT_MSG(false, "print should support stringFormat or single string");
             }
         }
 
@@ -485,6 +613,12 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
             for(size_t i = 0; i < x.n_args; i++) {
                 ASR::expr_t* value = x.m_args[i];
                 if (ASR::is_a<ASR::ImpliedDoLoop_t>(*value)) {
+                    ASR::ImpliedDoLoop_t* implied_do_loop = ASR::down_cast<ASR::ImpliedDoLoop_t>(value);
+                    if ( ASR::is_a<ASR::Tuple_t>(*implied_do_loop->m_type) ) {
+                        remove_original_statement = true;
+                        pass_result.push_back(al, create_do_loop_form_idl(implied_do_loop));
+                        continue;
+                    }
                     ASR::asr_t* array_constant = create_array_constant(x, value);
                     string_format_stmt->m_args[i] = ASRUtils::EXPR(array_constant);
 
@@ -509,7 +643,24 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
             }
         }
 
+        void visit_FileRead(const ASR::FileRead_t &x) {
+            if (x.m_overloaded) {
+                this->visit_stmt(*x.m_overloaded);
+                remove_original_statement = false;
+                return;
+            }
+        }
+
         void visit_FileWrite(const ASR::FileWrite_t &x) {
+            file_write = true;
+            m_unit = x.m_unit;
+            if (x.m_overloaded) {
+                this->visit_stmt(*x.m_overloaded);
+                remove_original_statement = false;
+                file_write = false;
+                return;
+            }
+
             /*
                 integer :: i
                 write(*,*) (i, i=1, 10)
@@ -546,6 +697,7 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                     }
                 }
             }
+            file_write = false;
         }
 
         void visit_CPtrToPointer(const ASR::CPtrToPointer_t& x) {
