@@ -660,9 +660,9 @@ public:
                         t = ASRUtils::TYPE(ASR::make_Real_t(al, t->base.loc, tnew->m_kind));
                         break;
                     }
-                    case ASR::ttypeType::Character: {
-                        ASR::Character_t* tnew = ASR::down_cast<ASR::Character_t>(t);
-                        t = ASRUtils::TYPE(ASR::make_Character_t(al, t->base.loc,
+                    case ASR::ttypeType::String: {
+                        ASR::String_t* tnew = ASR::down_cast<ASR::String_t>(t);
+                        t = ASRUtils::TYPE(ASR::make_String_t(al, t->base.loc,
                                     tnew->m_kind, tnew->m_len, tnew->m_len_expr, ASR::string_physical_typeType::PointerString));
                         break;
                     }
@@ -719,7 +719,7 @@ public:
             }
             case (ASR::ttypeType::Allocatable): {
                 ASR::Allocatable_t *a = ASR::down_cast<ASR::Allocatable_t>(ttype);
-                return ASRUtils::TYPE(ASR::make_Allocatable_t(al, ttype->base.loc,
+                return ASRUtils::TYPE(ASRUtils::make_Allocatable_t_util(al, ttype->base.loc,
                     substitute_type(a->m_type)));
             }
             case (ASR::ttypeType::ClassType): {
@@ -1389,7 +1389,7 @@ public:
             }
             case (ASR::ttypeType::Allocatable) : {
                 ASR::Allocatable_t *a = ASR::down_cast<ASR::Allocatable_t>(ttype);
-                return ASRUtils::TYPE(ASR::make_Allocatable_t(al, ttype->base.loc,
+                return ASRUtils::TYPE(ASRUtils::make_Allocatable_t_util(al, ttype->base.loc,
                     substitute_type(a->m_type)));
             }
             case (ASR::ttypeType::ClassType) : {
@@ -1809,7 +1809,7 @@ public:
             }
             case (ASR::ttypeType::Allocatable): {
                 ASR::Allocatable_t *a = ASR::down_cast<ASR::Allocatable_t>(ttype);
-                return ASRUtils::TYPE(ASR::make_Allocatable_t(al, ttype->base.loc,
+                return ASRUtils::TYPE(ASRUtils::make_Allocatable_t_util(al, ttype->base.loc,
                     substitute_type(a->m_type)));
             }
             case (ASR::ttypeType::ClassType) : {
