@@ -59,6 +59,7 @@ struct PassOptions {
     bool with_intrinsic_mods = false;
     bool c_mangling = false;
     bool openmp = false;
+    bool experimental_simplifier = false;
 };
 
 struct CompilerOptions {
@@ -86,6 +87,7 @@ struct CompilerOptions {
     bool lookup_name = false;
     std::string line = "";
     std::string column = "";
+    bool continue_compilation = false;
     bool generate_object_code = false;
     bool no_warnings = false;
     bool disable_style = false;
